# -*- coding: utf-8 -*-

from lamvery.actions.base import BaseAction


class EncryptFileAction(BaseAction):

    def __init__(self, args):
        super(EncryptFileAction, self).__init__(args)
        self._file = args.file
        self._path = args.path
        self._store = args.store

    def action(self):
        cipher_text = self.get_kms_client().encrypt(
            self._config.get_secret().get('key_id'), open(self._path, 'r').read())

        if self._store:
            self._config.store_secret_file(self._file, cipher_text)
        else:
            print(cipher_text)



# -*- coding: utf-8 -*-

from lamvery.actions.base import BaseAction
from lamvery.utils import confirm_overwrite

PY_CODE = """import lamvery

def {}(event, context):
    # Use environment variables
    # lamvery.env.load()
    # print(os.environ['FOO'])

    # Use KMS encryption
    # print(lamvery.secret.get('foo'))

    print('This is a skeleton function.')
"""

JS_CODE = """var lamvery = require('./lamvery.js');

exports.{} = function(event, context) {{
    // Use environment variables
    // lamvery.env.load();
    // console.log(process.env.FOO);

    // Use KMS encryption
    // lamvery.secret.get('foo', function(err, data) {{
    //     console.log(data);
    // }});

    console.log('This is a skeleton function.');
}}
"""


class GenerateAction(BaseAction):

    def __init__(self, args):
        super(GenerateAction, self).__init__(args)
        self._kind = args.kind

    def action(self):
        if self._kind == 'function':
            self._generate_function(
                self._config.get_handler_namespace(),
                self._config.get_handler_function(),
                self._config.get_runtime())
        else:
            raise Exception('"{}" kind is not supported.'.format(self._kind))

    def _generate_function(self, namespace, function, runtime):
        if 'python' in runtime:
            path = '{}.py'.format(namespace)
            content = PY_CODE.format(function)
        elif 'nodejs' in runtime:
            path = '{}.js'.format(namespace)
            content = JS_CODE.format(function)
        else:
            raise Exception('Runtime "{}" is not supported.'.format(runtime))

        if confirm_overwrite(path):
            open(path, 'w').write(content)
            self._logger.info('Output skeleton function: {}'.format(path))



# -*- coding: utf-8 -*-

from lamvery.actions.base import BaseAction

CONF_DIFF_KEYS = [
    ('Runtime', 'runtime',),
    ('Role', 'role',),
    ('Handler', 'handler',),
    ('Description', 'description',),
    ('Timeout', 'timeout',),
    ('MemorySize', 'memory_size',),
]

VPC_DIFF_KEYS = [
    ('SubnetIds', 'subnets',),
    ('SecurityGroupIds', 'security_groups',),
]

class ConfigureAction(BaseAction):

    def __init__(self, args):
        super(ConfigureAction, self).__init__(args)

    def action(self):
        func_name = self._config.get_function_name()
        local_conf = self._config.get_configuration()
        client = self.get_lambda_client()
        remote_conf = client.get_function_conf(func_name)
        vpc_config = self._config.get_vpc_configuration()

        if len(remote_conf) > 0:
            self._print_diff(
                prefix='[Function]',
                remote=remote_conf, local=local_conf, keys=CONF_DIFF_KEYS)

            if vpc_config is not None:
                self._print_diff(
                    prefix='[Function]',
                    remote=remote_conf.get('VpcConfig', {}), local=vpc_config, keys=VPC_DIFF_KEYS)

            client.update_function_conf(local_conf)
        else:
            msg = '"{}" function is not exists. Please `deploy` at first.'.format(func_name)
            raise Exception(msg)



# -*- coding: utf-8 -*-

import json
import re
import hashlib

from datetime import datetime
from datadiff import diff
from lamvery.actions.base import BaseAction
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import TerminalFormatter

DEFAULT_MAPPING_TEMPLATE = '''
##  See http://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html
##  This template will pass through all parameters including path, querystring, header, stage variables,
##  and context through to the integration endpoint via the body/payload
#set($allParams = $input.params())
{
"body-json" : "$input.json('$')",
"params" : {
#foreach($type in $allParams.keySet())
    #set($params = $allParams.get($type))
"$type" : {
    #foreach($paramName in $params.keySet())
    "$paramName" : "$util.escapeJavaScript($params.get($paramName))"
        #if($foreach.hasNext),#end
    #end
}
    #if($foreach.hasNext),#end
#end
},
"stage-variables" : {
#foreach($key in $stageVariables.keySet())
"$key" : "$util.escapeJavaScript($stageVariables.get($key))"
    #if($foreach.hasNext),#end
#end
},
"context" : {
    "account-id" : "$context.identity.accountId",
    "api-id" : "$context.apiId",
    "api-key" : "$context.identity.apiKey",
    "authorizer-principal-id" : "$context.authorizer.principalId",
    "caller" : "$context.identity.caller",
    "cognito-authentication-provider" : "$context.identity.cognitoAuthenticationProvider",
    "cognito-authentication-type" : "$context.identity.cognitoAuthenticationType",
    "cognito-identity-id" : "$context.identity.cognitoIdentityId",
    "cognito-identity-pool-id" : "$context.identity.cognitoIdentityPoolId",
    "http-method" : "$context.httpMethod",
    "stage" : "$context.stage",
    "source-ip" : "$context.identity.sourceIp",
    "user" : "$context.identity.user",
    "user-agent" : "$context.identity.userAgent",
    "user-arn" : "$context.identity.userArn",
    "request-id" : "$context.requestId",
    "resource-id" : "$context.resourceId",
    "resource-path" : "$context.resourcePath"
    }
}
'''


class ApiAction(BaseAction):

    def __init__(self, args):
        super(ApiAction, self).__init__(args)
        self._no_integrate = args.no_integrate
        self._remove = args.remove
        self._write_id = args.write_id
        self._stage = args.stage

    def get_stage_name(self):
        if self._stage is not None:
            return self._stage
        else:
            return self._config.get_api_stage()

    def get_cors(self):
        cors = self._config.get_api_cors()
        if cors is not None:
            cors['origin'] = "'{}'".format(cors['origin'])
            cors['methods'] = "'{}'".format(','.join(cors['methods']))
            cors['headers'] = "'{}'".format(','.join(cors['headers']))
        return cors

    def action(self):
        client = self.get_apigateway_client()
        api_id = self._config.get_api_id()
        stage = self.get_stage_name()
        cors = self.get_cors()
        api_conf = self._config.get_api_configuration()

        if not self._no_integrate:
            api_conf = self._integrate_aws(api_conf, stage, cors)

        self._print_conf_diff(
            api_conf, self._get_remote_configuration(client, api_id, stage))

        if not self._dry_run:
            ret = self._apply_api(client, api_id, api_conf)
            if api_id is None:
                api_id = ret['id']

            if self._write_id:
                self._config.save_api_id(ret['id'])

            self._add_permissions(api_id, api_conf)

            self._print_apply_result(ret)
            ret = self._deploy(client, ret['id'], stage)
            self._print_deploy_result(ret)

    def _add_permissions(self, api_id, api_conf):
        client = self.get_lambda_client()

        for path, methods in api_conf['paths'].items():
            for method in methods.keys():
                client.add_permission(
                    self._config.get_function_name(),
                    self.get_alias_name(),
                    hashlib.md5(api_id + method + path).hexdigest(),
                    'arn:aws:execute-api:{}:{}:{}/*/{}{}'.format(
                        client._session.region_name,
                        client.get_account_id(),
                        api_id,
                        method.upper(),
                        path),
                    'apigateway.amazonaws.com')

    def _get_remote_configuration(self, client, api_id, stage):
        if api_id is None:
            return {}
        return client.get_export(api_id, stage)

    def _integrate_aws(self, api_conf, stage, cors):
        client = self.get_lambda_client()
        api_conf['basePath'] = '/{}'.format(stage)
        api_conf['info']['version'] = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')

        for path, methods in api_conf['paths'].items():

            for method, conf in methods.items():
                conf.setdefault('x-amazon-apigateway-integration', {})
                integration = conf['x-amazon-apigateway-integration']
                integration.setdefault('type', 'aws')
                integration.setdefault('httpMethod', 'POST')

                uri = 'arn:aws:apigateway:{}:lambda:path/2015-03-31/functions/'.format(
                    client._session.region_name)
                uri += '{}/invocations'.format(client.get_function_conf(
                    self._config.get_function_name(),
                    self.get_alias_name()).get('FunctionArn'))

                integration.setdefault('uri', uri)
                integration.setdefault(
                    'requestTemplates',
                    {'application/json': DEFAULT_MAPPING_TEMPLATE})

                for code in conf['responses'].keys():
                    integration.setdefault('responses', {
                        'default': {
                            'statusCode': str(code)
                        }
                    })

                    if cors is not None:
                        conf['responses'][code].setdefault('headers', {
                            'Access-Control-Allow-Origin': {'type': 'string'}
                        })

                if cors is not None:
                    integration['responses']['default'].setdefault(
                        'responseParameters', self._generate_method_cors(cors))

                    api_conf['paths'][path].setdefault(
                        'options',
                        self._generate_option_cors(cors))

                conf['x-amazon-apigateway-integration'] = integration
                api_conf['paths'][path][method] = conf

        return api_conf

    def _generate_method_cors(self, cors):
        return {
            'method.response.header.Access-Control-Allow-Origin': cors.get('origin')
        }

    def _generate_option_cors(self, cors):
        return {
            'consumes': ['application/json'],
            'produces': ['application/json'],
            'responses': {
                200: {
                    'description': '200 response',
                    'headers': {
                        'Access-Control-Allow-Origin': {'type': 'string'},
                        'Access-Control-Allow-Methods': {'type': 'string'},
                        'Access-Control-Allow-Headers': {'type': 'string'},
                    }
                }
            },
            'x-amazon-apigateway-integration': {
                'responses': {
                    'default': {
                        'statusCode': '200',
                        'responseParameters': {
                            'method.response.header.Access-Control-Allow-Methods': cors.get('methods'),
                            'method.response.header.Access-Control-Allow-Headers': cors.get('headers'),
                            'method.response.header.Access-Control-Allow-Origin': cors.get('origin'),
                        }
                    }
                },
                'requestTemplates': {
                    'application/json': '{"statusCode": 200}'
                },
                'type': 'mock'
            }
        }

    def _apply_api(self, client, api_id, api_conf):
        api = client.get_rest_api(api_id)
        ret = None

        if api is None:
            if self._remove:
                self._logger.info('"{}" is not exists.'.format(api_id))
            else:
                self._logger.info('[API] Add new rest api.')
                ret = client.import_rest_api(api_conf)
        else:
            if self._remove:
                self._logger.warn('[API] Delete rest api "{}".'.format(api_id))
                client.delete_rest_api(api_id)
            else:
                self._logger.info('[API] Update rest api "{}".'.format(api_id))
                ret = client.put_rest_api(api_id, api_conf)

        return ret

    def _print_apply_result(self, ret):
        if ret is not None:

            self._logger.info('[API] ID: {}'.format(ret['id']))
            self._logger.info('[API] Name: {}'.format(ret['name']))
            self._logger.info('[API] Description: {}'.format(ret.get('description')))

            for w in ret.get('warnings', []):
                self._logger.warn('[API] Warning: "{}".'.format(w))

    def _deploy(self, client, api_id, stage):
        return client.create_deployment(api_id, stage)

    def _print_deploy_result(self, ret):
        if ret is not None:
            self._logger.info('[API] Deployment ID: {}'.format(ret['id']))
            self._logger.info('[API] Deployment description: {}'.format(ret.get('description')))
            self._logger.info('[API] Deployment API Summary: {}'.format(ret.get('apiSummary')))

    def _print_conf_diff(self, local, remote):
        str_diff = diff(remote, json.loads(json.dumps(local))).__str__()
        str_diff = re.compile(r'^( +)([\-\+])', re.MULTILINE).sub(r'\2\1', str_diff)
        self._logger.warn('[API] Configuration:\n{}'.format(highlight(
            str_diff,
            get_lexer_by_name('diff'),
            TerminalFormatter())))



# -*- coding: utf-8 -*-

import base64

from lamvery.clients.base import BaseClient


class KmsClient(BaseClient):

    def __init__(self, *args, **kwargs):
        super(KmsClient, self).__init__(*args, **kwargs)
        self._kms = self._session.client('kms')

    def encrypt(self, key_id, text):
        res = self._kms.encrypt(KeyId=key_id, Plaintext=text)
        return base64.b64encode(res.get('CiphertextBlob'))

    def decrypt(self, cipher_text):
        res = self._kms.decrypt(CiphertextBlob=base64.b64decode(cipher_text))
        return res.get('Plaintext')



# -*- coding: utf-8 -*-

import json
import os

ENV_FILE_NAME = '.lamvery_env.json'


def load():
    try:
        env = json.load(open(ENV_FILE_NAME, 'r'))
        for k, v in env.items():
            os.environ.setdefault(k, v)

    except:
        pass



# -*- coding: utf-8 -*-

import sys
import argparse
import lamvery
import logging
from termcolor import colored
from lamvery.actions import (
    InitAction,
    BuildAction,
    ConfigureAction,
    DeployAction,
    DecryptAction,
    EncryptAction,
    EncryptFileAction,
    EventsAction,
    InvokeAction,
    RollbackAction,
    SetAliasAction,
    LogsAction,
    ApiAction,
    GenerateAction
)


def init(args):
    InitAction(args).action()


def build(args):
    BuildAction(args).action()


def configure(args):
    ConfigureAction(args).action()


def deploy(args):
    DeployAction(args).action()


def decrypt(args):
    DecryptAction(args).action()


def encrypt(args):
    EncryptAction(args).action()


def encrypt_file(args):
    EncryptFileAction(args).action()


def events(args):
    EventsAction(args).action()


def generate(args):
    GenerateAction(args).action()


def invoke(args):
    InvokeAction(args).action()


def rollback(args):
    RollbackAction(args).action()


def set_alias(args):
    SetAliasAction(args).action()


def logs(args):
    LogsAction(args).action()


def api(args):
    ApiAction(args).action()


def main():
    alias_args = ('-a', '--alias',)
    alias_kwargs = {
        'help': 'Alias for a version of the function',
        'default': None
    }
    conf_file_args = ('-c', '--conf-file',)
    conf_file_kwargs = {
        'help': 'Configuration YAML file (default: .lamvery.yml)',
        'default': '.lamvery.yml'
    }
    dry_run_args = ('-d', '--dry-run',)
    dry_run_kwargs = {
        'help': 'Dry run',
        'action': 'store_true',
        'default': False
    }
    keep_empty_args = ('-k', '--keep-empty-events',)
    keep_empty_kwargs = {
        'help': 'Keep the event rules that does not have any targets.',
        'action': 'store_true',
        'default': False
    }
    single_file_args = ('-s', '--single-file',)
    single_file_kwargs = {
        'help': 'Only use the main lambda function file',
        'action': 'store_true',
        'default': False
    }
    no_libs_args = ('-l', '--no-libs',)
    no_libs_kwargs = {
        'help': 'Archiving without all libraries',
        'action': 'store_true',
        'default': False
    }
    secret_name_args = ('-n', '--secret-name',)
    secret_name_kwargs = {
        'help': 'The name of the secret value',
        'default': None
    }
    publish_args = ('-p', '--publish')
    publish_kwargs = {
        'help': 'Publish the version as an atomic operation',
        'action': 'store_true',
        'default': False
    }
    store_args = ('-s', '--store',)
    store_kwargs = {
        'help': 'Store encripted value to the configuration file (default: .lamvery.secret.yml)',
        'action': 'store_true',
        'default': False
    }
    version_args = ('-v', '--version',)
    version_kwargs = {
        'help': 'Version of the function',
        'default': None
    }
    follow_args = ('-f', '--follow',)
    follow_kwargs = {
        'help': 'Watch the log events and updates the display (like `tail -f`)',
        'action': 'store_true',
        'default': False
    }
    filter_args = ('-F', '--filter',)
    filter_kwargs = {
        'help': 'Filtering pattern for the log messages',
        'default': None
    }
    interval_args = ('-i', '--interval',)
    interval_kwargs = {
        'help': 'Intervals(seconds) to watch the log events',
        'default': 1
    }
    start_args = ('-s', '--start',)
    start_kwargs = {
        'help': 'Time to start the log events watching',
        'default': None
    }
    target_args = ('-t', '--target',)
    target_kwargs = {
        'help': 'The alias of the version that is targeted for setting alias',
        'default': None
    }
    env_args = ('-e', '--env',)
    env_kwargs = {
        'help': 'Environment variables that pass to the function',
        'action': 'append',
        'default': None
    }
    write_args = ('-w', '--write-id',)
    write_kwargs = {
        'help': 'Write the id of your API to the configuration file (default: .lamvery.api.yml)',
        'action': 'store_true',
        'default': False
    }
    stage_args = ('-s', '--stage',)
    stage_kwargs = {
        'help': 'The name of the stage in API Gateway',
        'default': None
    }
    remove_args = ('-r', '--remove',)
    remove_kwargs = {
        'help': 'Remove your API',
        'action': 'store_true',
        'default': False
    }
    no_integrate_args = ('-n', '--no-integrate',)
    no_integrate_kwargs = {
        'help': 'Without automatic integration',
        'action': 'store_true',
        'default': False
    }
    kind_args = ('-k', '--kind',)
    kind_kwargs = {
        'help': 'The kind of the file # accepts "function"',
        'required': True
    }
    filename_args = ('-n', '--name',)
    filename_kwargs = {
        'help': 'The filename to put the decrypted file in the function environment',
        'required': True
    }

    parser = argparse.ArgumentParser(
        description='Yet another deploy tool for AWS Lambda in the virtualenv environment.',
        epilog='Lamvery version: {}'.format(lamvery.__version__))
    subparsers = parser.add_subparsers(title='subcommands')

    init_parser = subparsers.add_parser(
        'init',
        help='Generate initial configuration file')
    init_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    init_parser.set_defaults(func=init)

    build_parser = subparsers.add_parser(
        'build',
        help='Build and archive your code and libraries to <your-function-name>.zip')
    build_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    build_parser.add_argument(*single_file_args, **single_file_kwargs)
    build_parser.add_argument(*no_libs_args, **no_libs_kwargs)
    build_parser.add_argument(*env_args, **env_kwargs)
    build_parser.set_defaults(func=build)

    set_alias_parser = subparsers.add_parser(
        'set-alias',
        help='Set alias to a version of the function')
    set_alias_parser.add_argument(*alias_args, **alias_kwargs)
    set_alias_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    set_alias_parser.add_argument(*dry_run_args, **dry_run_kwargs)
    set_alias_parser.add_argument(*version_args, **version_kwargs)
    set_alias_parser.add_argument(*target_args, **target_kwargs)
    set_alias_parser.set_defaults(func=set_alias)

    configure_parser = subparsers.add_parser(
        'configure',
        help='Update the remote configuration')
    configure_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    configure_parser.add_argument(*dry_run_args, **dry_run_kwargs)
    configure_parser.set_defaults(func=configure)

    deploy_parser = subparsers.add_parser(
        'deploy',
        help='Deploy your code and libraries,' +
             'Update the remote configuration, Set alias (optional)')
    deploy_parser.add_argument(*alias_args, **alias_kwargs)
    deploy_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    deploy_parser.add_argument(*dry_run_args, **dry_run_kwargs)
    deploy_parser.add_argument(*single_file_args, **single_file_kwargs)
    deploy_parser.add_argument(*no_libs_args, **no_libs_kwargs)
    deploy_parser.add_argument(*publish_args, **publish_kwargs)
    deploy_parser.add_argument(*env_args, **env_kwargs)
    deploy_parser.set_defaults(func=deploy)

    encrypt_parser = subparsers.add_parser('encrypt', help='Encrypt a text value using KMS')
    encrypt_parser.add_argument('text', help='The text value to encrypt')
    encrypt_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    encrypt_parser.add_argument(*secret_name_args, **secret_name_kwargs)
    encrypt_parser.add_argument(*store_args, **store_kwargs)
    encrypt_parser.set_defaults(func=encrypt)

    decrypt_parser = subparsers.add_parser('decrypt', help='Decrypt the secret value using KMS')
    decrypt_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    decrypt_parser.add_argument(*secret_name_args, **secret_name_kwargs)
    decrypt_parser.set_defaults(func=decrypt)

    encrypt_file_parser = subparsers.add_parser('encrypt-file', help='Encrypt a file using KMS')
    encrypt_file_parser.add_argument('path', help='The file path to encrypt')
    encrypt_file_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    encrypt_file_parser.add_argument(*filename_args, **filename_kwargs)
    encrypt_file_parser.add_argument(*store_args, **store_kwargs)
    encrypt_file_parser.set_defaults(func=encrypt_file)

    decrypt_parser = subparsers.add_parser('decrypt', help='Decrypt the secret value using KMS')
    decrypt_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    decrypt_parser.add_argument(*secret_name_args, **secret_name_kwargs)
    decrypt_parser.set_defaults(func=decrypt)

    events_parser = subparsers.add_parser(
        'events',
        help='Configure all events of CloudWatchEvents using the function')
    events_parser.add_argument(*alias_args, **alias_kwargs)
    events_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    events_parser.add_argument(*dry_run_args, **dry_run_kwargs)
    events_parser.add_argument(*keep_empty_args, **keep_empty_kwargs)
    events_parser.set_defaults(func=events)

    invoke_parser = subparsers.add_parser(
        'invoke',
        help='Invoke the function')
    invoke_parser.add_argument(
        'json', default='{}', help='The JSON string or file that pass to the function')
    invoke_parser.add_argument(*alias_args, **alias_kwargs)
    invoke_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    invoke_parser.add_argument(*version_args, **version_kwargs)
    invoke_parser.set_defaults(func=invoke)

    rollback_parser = subparsers.add_parser(
        'rollback',
        help='Rollback your code and libraries')
    rollback_parser.add_argument(*alias_args, **alias_kwargs)
    rollback_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    rollback_parser.add_argument(*version_args, **version_kwargs)
    rollback_parser.set_defaults(func=rollback)

    logs_parser = subparsers.add_parser(
        'logs',
        help="Watch the function's log events on CloudWatch Logs")
    logs_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    logs_parser.add_argument(*follow_args, **follow_kwargs)
    logs_parser.add_argument(*filter_args, **filter_kwargs)
    logs_parser.add_argument(*interval_args, **interval_kwargs)
    logs_parser.add_argument(*start_args, **start_kwargs)
    logs_parser.set_defaults(func=logs)

    api_parser = subparsers.add_parser(
        'api',
        help='Manage your APIs')
    api_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    api_parser.add_argument(*dry_run_args, **dry_run_kwargs)
    api_parser.add_argument(*no_integrate_args, **no_integrate_kwargs)
    api_parser.add_argument(*remove_args, **remove_kwargs)
    api_parser.add_argument(*stage_args, **stage_kwargs)
    api_parser.add_argument(*write_args, **write_kwargs)
    api_parser.set_defaults(func=api)

    gen_parser = subparsers.add_parser(
        'generate',
        help='Ganerate skeleton files')
    gen_parser.add_argument(*conf_file_args, **conf_file_kwargs)
    gen_parser.add_argument(*kind_args, **kind_kwargs)
    gen_parser.set_defaults(func=generate)

    try:
        args = parser.parse_args()
        args.func(args)
        sys.exit(0)
    except Exception as e:
        logging.exception(e)
        sys.exit(colored(str(e), 'red'))



# -*- coding: utf-8 -*-

from lamvery.actions.base import BaseAction
from lamvery.build import Builder
from lamvery.utils import parse_env_args


class BuildAction(BaseAction):

    def __init__(self, args):
        super(BuildAction, self).__init__(args)
        self._single_file = args.single_file
        self._no_libs = args.no_libs
        self._env = parse_env_args(args.env)

    def action(self):
        archive_name = self._config.get_archive_name()
        function_filename = self._config.get_function_filename()
        secret = self._config.generate_lambda_secret()
        exclude = self._config.get_exclude()

        builder = Builder(
            filename=archive_name,
            function_filename=function_filename,
            secret=secret,
            single_file=self._single_file,
            no_libs=self._no_libs,
            exclude=exclude,
            runtime=self._config.get_runtime(),
            env=self._env,
            clean_build=self._config.is_clean_build(),
            hooks=self._config.get_build_hooks())

        zipfile = builder.build()
        with open(archive_name, 'w') as f:
            f.write(zipfile.read())
        zipfile.close()

        self._logger.info('Output archive(zip) to {}'.format(archive_name))



# -*- coding: utf-8 -*-

from lamvery.actions.base import BaseAction


class SetAliasAction(BaseAction):

    def __init__(self, args):
        super(SetAliasAction, self).__init__(args)

        if hasattr(args, 'target'):
            self._target = args.target
        else:
            self._target = None

        if hasattr(args, 'version'):
            self._version = args.version
        else:
            self._version = None

    def action(self):
        alias_name = self.get_alias_name()
        func_name = self._config.get_function_name()
        version = self.get_version(func_name)
        client = self.get_lambda_client()

        if alias_name is None:
            raise Exception(
                'Please specify an alias by `-a` option or `default_alias` configuration.')

        current_alias = client.get_alias(func_name, alias_name)
        self._print_alias_diff(alias_name, current_alias, version)

        if len(current_alias) > 0:
            client.update_alias(func_name, alias_name, version)
        else:
            client.create_alias(func_name, alias_name, version)

    def _print_alias_diff(self, name, current, version):
        self._logger.warn(
            '[Alias] {name}: {cur} -> {new}'.format(
                name=name, cur=current.get('FunctionVersion'), new=version))

    def get_version(self, function):
        version = '$LATEST'

        if self._version is not None:
            version = self._version

        elif self._target is not None:
            target_alias = self.get_lambda_client().get_alias(function, self._target)
            version = target_alias.get('FunctionVersion')

            if version is None:
                raise Exception(
                    'Target alias "{}" is not exists in "{}" function.'.format(
                        self._target, function))

        return version



from lamvery.actions.build import BuildAction
from lamvery.actions.configure import ConfigureAction
from lamvery.actions.decrypt import DecryptAction
from lamvery.actions.deploy import DeployAction
from lamvery.actions.encrypt import EncryptAction
from lamvery.actions.encrypt_file import EncryptFileAction
from lamvery.actions.events import EventsAction
from lamvery.actions.init import InitAction
from lamvery.actions.set_alias import SetAliasAction
from lamvery.actions.invoke import InvokeAction
from lamvery.actions.rollback import RollbackAction
from lamvery.actions.logs import LogsAction
from lamvery.actions.api import ApiAction
from lamvery.actions.generate import GenerateAction



from lamvery.clients.events import EventsClient
from lamvery.clients.kms import KmsClient
from lamvery.clients.function import LambdaClient
from lamvery.clients.logs import LogsClient
from lamvery.clients.apigateway import ApiGatewayClient



# -*- coding: utf-8 -*-

import yaml
import os
import re

from collections import OrderedDict
from jinja2 import Environment, FileSystemLoader
from jinja2.exceptions import TemplateNotFound
from lamvery.log import get_logger

RUNTIME_PY_27 = 'python2.7'
RUNTIME_NODE_JS = 'nodejs'

RUNTIME_AND_EXTENSION = {
    RUNTIME_PY_27: 'py',
    RUNTIME_NODE_JS: 'js'
}


def represent_odict(dumper, instance):
    return dumper.represent_mapping(u'tag:yaml.org,2002:map', instance.items())

yaml.add_representer(OrderedDict, represent_odict)
yaml.add_constructor(
    yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
    lambda loader, node: OrderedDict(loader.construct_pairs(node)))


class Config:

    def __init__(self, conf_file):
        self._file = conf_file
        self._template_env = Environment(loader=FileSystemLoader('./', encoding='utf8'))

    def load(self, file, default={}):
        try:
            tmpl = self._template_env.get_template(file)
            return yaml.load(tmpl.render({'env': os.environ}))
        except TemplateNotFound:
            get_logger(__name__).warn(
                'Configuration file "{}" is not exists.'.format(file))
            return default

    def load_conf(self):
        return self.load(self._file)

    def load_events(self):
        return self.load(self.get_event_file())

    def get_event_file(self):
        return self.load_conf().get('event_file', '.lamvery.event.yml')

    def load_secret(self):
        return self.load(self.get_secret_file())

    def get_secret_file(self):
        return self.load_conf().get('secret_file', '.lamvery.secret.yml')

    def load_exclude(self):
        return self.load(self.get_exclude_file(), [])

    def get_exclude_file(self):
        return self.load_conf().get('exclude_file', '.lamvery.exclude.yml')

    def load_hook(self):
        return self.load(self.get_hook_file())

    def get_hook_file(self):
        return self.load_conf().get('hook_file', '.lamvery.hook.yml')

    def load_api(self):
        return self.load(self.get_api_file())

    def get_api_file(self):
        return self.load_conf().get('api_file', '.lamvery.api.yml')

    def load_raw(self, file_name):
        txt = open(file_name, 'r').read()
        return yaml.load(self.escape(txt))

    def load_raw_secret(self):
        return self.load_raw(self.get_secret_file())

    def load_raw_api(self):
        return self.load_raw(self.get_api_file())

    def escape(self, txt):
        txt = txt.replace("'", "$$")
        txt = txt.replace(
            self._template_env.variable_start_string,
            "'" + self._template_env.variable_start_string)
        txt = txt.replace(
            self._template_env.block_start_string,
            "'" + self._template_env.block_start_string)
        txt = txt.replace(
            self._template_env.variable_end_string,
            self._template_env.variable_end_string + "'")
        txt = txt.replace(
            self._template_env.block_end_string,
            self._template_env.block_end_string + "'")
        return txt

    def unescape(self, txt):
        txt = txt.replace("$$", "'")
        txt = txt.replace(
            "'" + self._template_env.variable_start_string,
            self._template_env.variable_start_string)
        txt = txt.replace(
            "'" + self._template_env.block_start_string,
            self._template_env.block_start_string)
        txt = txt.replace(
            self._template_env.variable_end_string + "'",
            self._template_env.variable_end_string)
        txt = txt.replace(
            self._template_env.block_end_string + "'",
            self._template_env.block_end_string)
        return txt

    def get_configuration(self):
        return self.load_conf().get('configuration')

    def get_vpc_configuration(self):
        vpc_config = self.get_configuration().get('vpc_config')

        if vpc_config is None:
            vpc_config = {}

        return {
            'subnets': vpc_config.get('subnets', []),
            'security_groups': vpc_config.get('security_groups', [])
        }

    def get_secret(self):
        return self.load_secret()

    def get_events(self):
        events = self.load_events()
        if events is None:
            return {'rules': []}

        if 'rules' in events:
            return {'rules': []}

        if isinstance(events, list):
            rules = []
            for e in events:
                e['name'] = e['rule']
                rules.append(e)

            return {'rules': rules}

        return events

    def get_default_alias(self):
        return self.load_conf().get('default_alias')

    def enable_versioning(self):
        return bool(self.load_conf().get('versioning'))

    def generate_lambda_secret(self):
        return {
            'region': self.get_region(),
            'cipher_texts': self.get_secret().get('cipher_texts'),
            'secret_files': self.get_secret().get('secret_files')
        }

    def get_function_name(self):
        if os.path.exists(self._file):
            return self.get_configuration().get('name')
        else:
            dirname = os.path.basename(os.getcwd())
            if dirname == '':
                return 'sample'
            return dirname

    def get_function_filename(self):
        ext = RUNTIME_AND_EXTENSION.get(self.get_runtime(), "py")
        return '{}.{}'.format(
            self.get_configuration().get('handler').split('.')[0], ext)

    def get_runtime(self):
        return self.get_configuration().get('runtime')

    def get_handler(self):
        return self.get_configuration().get('handler')

    def get_handler_namespace(self):
        return self.get_handler().split('.')[0]

    def get_handler_function(self):
        return self.get_handler().split('.')[1]

    def get_archive_name(self):
        return '{}.zip'.format(self.get_function_name())

    def get_region(self):
        if os.path.exists(self._file):
            return self.load_conf().get('region')
        else:
            return None

    def get_profile(self):
        return self.load_conf().get('profile')

    def get_exclude(self):
        exclude = self.load_exclude()
        if exclude is None:
            return []
        return exclude

    def get_build_hooks(self):
        return self.load_hook().get('build', {})

    def get_api_id(self):
        return self.load_api().get('api_id', None)

    def get_api_configuration(self):
        return self.load_api().get('configuration', {})

    def get_api_stage(self):
        return self.load_api().get('stage')

    def get_api_cors(self):
        return self.load_api().get('cors')

    def is_clean_build(self):
        return self.load_conf().get('clean_build', False)

    def get_default(self):
        init_vpc = OrderedDict()
        init_vpc['subnets'] = ['subnet-xxxxxxxx']
        init_vpc['security_groups'] = ['sg-xxxxxxxx']

        init_config = OrderedDict()
        init_config['name'] = self.get_function_name()
        init_config['runtime'] = RUNTIME_PY_27
        init_config['role'] = 'arn:aws:iam::<account-number>:role/<role>'
        init_config['handler'] = 'lambda_function.lambda_handler'
        init_config['description'] = 'This is a sample lambda function.'
        init_config['timeout'] = 10
        init_config['memory_size'] = 128
        init_config['vpc_config'] = init_vpc

        init_yaml = OrderedDict()
        init_yaml['profile'] = None
        init_yaml['region'] = 'us-east-1'
        init_yaml['versioning'] = False
        init_yaml['default_alias'] = None
        init_yaml['clean_build'] = False
        init_yaml['configuration'] = init_config

        return init_yaml

    def get_default_events(self):
        targets = [
            OrderedDict([
                ('id', '<unique-target-id>',),
                ('input', {'this': [{'is': 'a'}, {'sample': 'input'}]},),
                ('input_path', 'json.path.format',)])]
        event = OrderedDict()
        event['name'] = 'sample-rule-name'
        event['description'] = 'This is a sample CloudWatchEvent'
        event['schedule'] = 'rate(5 minutes)'
        event['targets'] = targets
        init_events = {'rules': [event]}

        return init_events

    def get_default_secret(self):
        init_secret = OrderedDict()
        init_secret['key_id'] = '<key-id>'
        init_secret['cipher_texts'] = OrderedDict()
        init_secret['secret_files'] = OrderedDict()

        return init_secret

    def get_default_exclude(self):
        return [
            '^{}$'.format(re.escape(self._file)),
            '^{}$'.format(re.escape(self.get_api_file())),
            '^{}$'.format(re.escape(self.get_hook_file())),
            '^{}$'.format(re.escape(self.get_event_file())),
            '^{}$'.format(re.escape(self.get_secret_file())),
            '^{}$'.format(re.escape(self.get_exclude_file()))]

    def get_default_hook(self):
        init_hook = OrderedDict()
        init_hook['build'] = OrderedDict()
        init_hook['build']['pre'] = []
        init_hook['build']['post'] = []

        return init_hook

    def get_default_api(self):
        init_api = OrderedDict()
        init_api['api_id'] = '<your-rest-api-id>'
        init_api['stage'] = 'dev'
        init_api['cors'] = OrderedDict([
            ('origin', '*',),
            ('methods', ['GET', 'OPTION'],),
            ('headers', ['Content-Type', 'X-Amz-Date', 'Authorization', 'X-Api-Key'],),
        ])
        init_api['configuration'] = OrderedDict([
            ('swagger', '2.0',),
            ('info', OrderedDict([
                ('title', 'Sample API',),
            ]),),
            ('schemes', ['https'],),
            ('paths', OrderedDict([
                ('/', OrderedDict([
                    ('get', OrderedDict([
                        ('produces', ['application/json'],),
                        ('parameters', [OrderedDict([
                            ('name', 'sample',),
                            ('in', 'query',),
                            ('required', False),
                            ('type', 'string',)])],),
                        ('responses', OrderedDict([
                            ('200', OrderedDict([
                                ('description', '200 response',),
                                ('schema', {'$ref': '#/definitions/Sample'},)])),
                        ]),),
                    ]),),
                ]),),
            ]),),
            ('definitions', OrderedDict([
                ('Sample', OrderedDict([
                    ('type', 'object',),
                ]),),
            ]),),
        ])

        return init_api

    def write(self, conf, path):
        txt = yaml.dump(
            conf,
            default_flow_style=False,
            allow_unicode=False)
        open(path, 'w').write(self.unescape(txt))

    def store_secret(self, key, value):
        conf = self.load_raw_secret()

        if 'cipher_texts' not in conf:
            conf['cipher_texts'] = {}

        conf['cipher_texts'][key] = value
        self.write(conf, self.get_secret_file())

    def store_secret_file(self, key, value):
        conf = self.load_raw_secret()

        if 'secret_files' not in conf:
            conf['secret_files'] = {}

        conf['secret_files'][key] = value
        self.write(conf, self.get_secret_file())

    def save_api_id(self, api_id):
        conf = self.load_raw_api()
        conf['api_id'] = str(api_id)
        self.write(conf, self.get_api_file())



# -*- coding: utf-8 -*-

from lamvery.actions.base import BaseAction
from lamvery.utils import confirm_overwrite


class InitAction(BaseAction):

    def __init__(self, args):
        super(InitAction, self).__init__(args)
        self._conf_file = args.conf_file

    def action(self):
        if confirm_overwrite(self._conf_file):
            self._config.write(self._config.get_default(), self._conf_file)
            self._logger.info(
                'Output initial file: {}'.format(self._conf_file))

        files = {
            self._config.get_event_file(): self._config.get_default_events(),
            self._config.get_secret_file(): self._config.get_default_secret(),
            self._config.get_exclude_file(): self._config.get_default_exclude(),
            self._config.get_hook_file(): self._config.get_default_hook(),
            self._config.get_api_file(): self._config.get_default_api(),
        }

        for f, c in files.items():
            if confirm_overwrite(f):
                self._config.write(c, f)
                self._logger.info(
                    'Output initial file: {}'.format(f))
